/*
 * debug - debug_io.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2005  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: debug_io.c,v 1.3 2005/07/07 06:39:40 krzysiek Exp $
 */

#include <unistd.h>
#include <debug_io.h>
#include <inttypes.h>
#include <errno.h>
#include <sys/io.h>

/* FIXME: ioperm and iopl are not portable */

static inline int d_io_enable(unsigned long port, int num)
{
	if (port + num < 1024)
		return ioperm(port, num, 1);
	return iopl(3);
}

static inline int d_io_disable(unsigned long port, int num)
{
	if (port + num < 1024)
		return ioperm(port, num, 0);
	return iopl(0);
}

static int d_inb_ioperm(unsigned long port, uint8_t *value)
{
	if (d_io_enable(port, 1))
		return -errno;
	*value = inb(port);
	if (d_io_disable(port, 1))
		return -errno;
	return 0;
}

static int d_outb_ioperm(unsigned long port, uint8_t value)
{
	if (d_io_enable(port, 1))
		return -errno;
	outb(value, port);
	if (d_io_disable(port, 1))
		return -errno;
	return 0;
}

static int d_inw_ioperm(unsigned long port, uint16_t *value)
{
	if (d_io_enable(port, 2))
		return -errno;
	*value = inw(port);
	if (d_io_disable(port, 2))
		return -errno;
	return 0;
}

static int d_outw_ioperm(unsigned long port, uint16_t value)
{
	if (d_io_enable(port, 2))
		return -errno;
	outw(value, port);
	if (d_io_disable(port, 2))
		return -errno;
	return 0;
}

static int d_inl_ioperm(unsigned long port, uint32_t *value)
{
	if (d_io_enable(port, 4))
		return -errno;
	*value = inl(port);
	if (d_io_disable(port, 4))
		return -errno;
	return 0;
}

static int d_outl_ioperm(unsigned long port, uint32_t value)
{
	if (d_io_enable(port, 4))
		return -errno;
	outl(value, port);
	if (d_io_disable(port, 4))
		return -errno;
	return 0;
}

int d_ioinit(void)
{
	d_inb = d_inb_ioperm;
	d_inw = d_inw_ioperm;
	d_inl = d_inl_ioperm;
	d_outb = d_outb_ioperm;
	d_outw = d_outw_ioperm;
	d_outl = d_outl_ioperm;
	return 0;
}
